﻿namespace TimelineOwnerHeaderWidth
{
    #region Using

    using System;
    using System.Data;
    using System.Drawing;
    using System.Windows.Forms;
    using Infragistics.Win;
    using Infragistics.Win.UltraWinSchedule;
    using Infragistics.Win.UltraWinSchedule.TimelineView;
    
    #endregion

    public partial class Form1 : Form, IUIElementDrawFilter
    {
        #region Declarations

        private bool _useDrawFilter = false;
        private DataTable _ownerTable = null;

        private string _bindKey = "Key";
        private string _bindName = "Name";
        private string _bindVisible = "Visible";
        private string _tableNameOwners = "Owners";

        #endregion

        #region Constructor

        public Form1()
        {
            InitializeComponent();
        }

        #endregion
        
        #region Form Events

        private void Form1_Load(object sender, EventArgs e)
        {
            this.TimelineView.AdditionalIntervals.Add(
                new Infragistics.Win.UltraWinSchedule.DateInterval()
                {
                    DateNavigationButtonVisibility = TimelineViewDateNavigationButtonVisibility.ShowOnFirstAndLastHeader
                });

            this.TimelineView.OwnerHeaderTextOrientation = new TextOrientationInfo(0, TextFlowDirection.Horizontal);
            this.TimelineView.DrawFilter = this;
            this.TimelineView.MaximumOwnersInView = 4;
            this.SetOwners();
        }

        private void checkBox1_CheckStateChanged(object sender, EventArgs e)
        {
            this._useDrawFilter = !this._useDrawFilter;
            this.SetOwners();
        }

        #endregion

        #region Owners

        private void SetOwners()
        {
            this.CalendarInfo.DataBindingsForOwners.DataSource = null;

            this.CalendarInfo.DataBindingsForOwners.BindingContextControl = this;
            this.CalendarInfo.DataBindingsForOwners.KeyMember = this._bindKey;
            this.CalendarInfo.DataBindingsForOwners.NameMember = this._bindName;
            this.CalendarInfo.DataBindingsForOwners.VisibleMember = this._bindVisible; 

            this.CalendarInfo.DataBindingsForOwners.DataSource = this.Owners;

            this.CalendarInfo.Owners.UnassignedOwner.Visible = false;
        }

        private DataTable Owners
        {
            get
            {
                this.CreateOwnersDataTable();
                this.CreateOwners(this._ownerTable);

                return this._ownerTable;
            }
        }

        private void CreateOwnersDataTable()
        {
            if (this._ownerTable == null)
            {
                this._ownerTable = new DataTable();
 
                this._ownerTable.Columns.Add(this._bindKey, typeof(string));
                this._ownerTable.Columns.Add(this._bindName, typeof(string));
                this._ownerTable.Columns.Add(this._bindVisible, typeof(bool));

                this._ownerTable.TableName = this._tableNameOwners;
            }
            else
            {
                this._ownerTable.Rows.Clear();
            }
        }

        private void CreateOwners(DataTable ownerTable)
        {
            ownerTable.Rows.Add(new object[] { "aaa", "Mr Edmund Blackadder", true });
            ownerTable.Rows.Add(new object[] { "bbb", "Mrs Miggins Pie Shop", true });
            ownerTable.Rows.Add(new object[] { "ccc", "The Baby Eating Bishop Of Bath And Wells", true });
            ownerTable.Rows.Add(new object[] { "ddd", "Captain Darling", true });
        }

        #endregion

        #region IUIElementDrawFilter Members

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            bool drawElement = false;

            if ((drawPhase == DrawPhase.BeforeDrawElement) && (drawParams.Element is TextUIElement))
            {
                drawElement = this.SplitOwnerHeaderText(ref drawParams);
            }

            return drawElement;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            if ((this._useDrawFilter) && (drawParams.Element is TextUIElement))
            {
                return DrawPhase.BeforeDrawElement;
            }
            else
            {
                return DrawPhase.None;
            }
        }

        #endregion

        #region Draw Filter Helper

        private bool SplitOwnerHeaderText(ref UIElementDrawParams drawParams)
        {
            // Check we have a text ui element
            TextUIElement textElement = drawParams.Element as TextUIElement;

            if (textElement != null)
            {
                // Check that this text ui element is a child of an owner header ui element
                OwnerHeaderUIElement headerElement = textElement.GetAncestor(typeof(OwnerHeaderUIElement)) as OwnerHeaderUIElement;

                if (headerElement != null)
                {
                    // Trim off any leading and trailing spaces
                    string text = headerElement.Owner.Name.Trim();
                    string space = " ";

                    // Determine how many spaces the text contains
                    int spaceCount = 0;
                    int spacePos = -1;

                    do
                    {
                        spaceCount += 1;
                        spacePos = text.IndexOf(space, spacePos + 1);
                    }
                    while (spacePos > 0);

                    // Count will always be 1 over
                    spaceCount -= 1;

                    // Determine which space to split the string on
                    int nthSpace = 0;

                    if (spaceCount > 0)
                    {
                        if ((spaceCount % 2) == 0)
                        {
                            // Even number of spaces
                            nthSpace = ((spaceCount / 2) + 1);
                        }
                        else
                        {
                            // Odd number of spaces
                            nthSpace = ((spaceCount + 1) / 2);
                        }

                        // Find the required nth space
                        spaceCount = 0;
                        spacePos = -1;

                        while (spaceCount < nthSpace)
                        {
                            spaceCount += 1;
                            spacePos = text.IndexOf(space, spacePos + 1);
                        }

                        // Divide the string at the required space
                        string line1 = text.Substring(0, spacePos);
                        string line2 = text.Substring((spacePos + 1), (text.Length - spacePos - 1));

                        // Make required adjustments to the text ui element
                        textElement.MultiLine = true;
                        textElement.Text = line1 + Environment.NewLine + line2;

                        Rectangle textRect = textElement.Rect;
                        textRect.Height *= 2;
                        textRect.Y = (headerElement.RectInsideBorders.Y + (headerElement.RectInsideBorders.Height / 2) - (textRect.Height / 2));
                        textElement.Rect = textRect;
                    }
                }
            }

            return false;
        }

        #endregion
    }
}
